{{
DataLogger100200_jsd.spin
Copyright (c) 2009 Steve Denson
See end of file for terms of use.                                       
This program builds on work done in or by the following:
  Data Logger Demo v0.050                                                 
  Author: Ray Rodrick                                                     
  Copyright (c) 2008 Ray Rodrick                                          
  See end of file for terms of use.                                       
  Ray's Acknowledgement: to Paul Baker's Logic Analyzer (dscope.spin) for the   
  inspiration and naming conventions.                                     
}}

{
CON
  _CLKMODE = XTAL1 + PLL16X     'Set to ext low-speed xtal, 16x PLL
  _XINFREQ = 5_000_000          'Xtal 5MHz

  BUFSZ = 512*4

VAR
'                     '\\       THE FOLLOWING MUST BE KEPT TOGETHER & IN THIS ORDER
  long  trigmask      '||       trigger pin mask (trigger immediately if =0)
  long  trigarm       '||       trigger pin arm state
  long  trigstate     '||       trigger pin capture state
  long  speriod       '||       sample period
  long  delay         '||       delay from trigger event to snap       
  long  slot          '||       number of cpu slots       
  long  syn           '||       startup sync cogs word
  long  bufsize[BUFNO]'||       dataset buffer size (set by cog when done)
  long  buffer[BUFSZ] '||       dataset buffer (max 512*slot, but not all used)
'                     '//       Note: This is not the same as dscope.spin !!!  
  
pub main

  repeat
    trigmask~ ' test only                     
    trigstate := trigstate & trigmask                     'enforce we don't look for masked pins (if error above 2 lines!)
    longfill(@buffer, 0, 512)                            'clear the buffer                  
    bufsize   := 0                                        'set by Cog when done to dataset size
    speriod   := 100

    start(@trigmask)
    repeat until bufsize

  repeat
'}

CON
  BUFNO = 4
  
VAR
  long  cog

PUB Start(bufptr)
'' Start DataLogger - starts a cog
'' returns 0 if no Cog available or cog "id"
''  bufptr = pointer to DL parameters
  stop
  result := cog := cognew(@entry, bufptr) + 1
  

PUB stop
'' Stop DataLogger - frees a cog
'' normally stops after sample completed and copied to buffer
''  num is cog number
  if cog
    cogstop(cog~ - 1)


DAT

'_______________________________________________________________________________
'
               org 0
entry          

  jmp          #init                          'go and setup the variables

'_______________________________________________________________________________
'
bufferfull
  mov          i,CogBufSz             'get actual buffer size
writeloop
  wrlong       bufstart,hptr            'write long to main memory (buffer)
  add          writeloop,d_inc          'increment buffer pointer
  add          hptr,#4                  'increment hub pointer by 4 long ... always use 4 cogs
  djnz         i,#writeloop             'repeat for entire buffer
  wrlong       CogBufSz,  bsize         'store size of dataset in bufsize (signals buffer written)

  cogid        i                        'get this Cog ID                       
  cogstop      i                        'stop this Cog

'_______________________________________________________________________________
'
prototype1     mov  bufstart-0,ina      'prototype "read all 32 pins" ... change to cnt for clkfreq sample proof

setuploop
  mov          bufstart-0,prototype1    'save the modified prototype instr
  add          setuploop,d_inc          'increment destination in instruction above
  add          prototype1,d_inc         'increment destination in prototype instruction
  djnz         i,#setuploop
  
'_______________________________________________________________________________
'
  cmp          del, #0 wz
  if_nz mov    jmpNoDelay, #0           'set nop ... no jump if delay              
  if_nz mov    trig, #0             'set nop ... no 2nd wait trigger if no delay              
   
  tjz          tmask,#nowait            'don't wait if =0
  cmp          tarm,  tmask  wz
  if_nz        waitpeq tarm,tmask       'wait for arm if arm != mask
   
jmpNoDelay
  jmp          #noDelay                 'will be nop if delay is used              
  waitpeq      tstate,tmask             'wait for start condition
  add          del, #$4
  add          del, cnt
  waitcnt      del, del                 'wait for delay*12.5ns to pass              

noDelay
trig   waitpeq tstate,tmask    'wait for start condition

nowait
                            
'_______________________________________________________________________________
'

bufstart       'registers and instructions from here overwritten by ina data

'_______________________________________________________________________________
'
init

  mov          hptr,par     'save hub memory pointer (initially set to trigmask)
  rdlong       tmask,hptr   'get triggermask parameter
  add          hptr,#4      'increment hub memory pointer     to trigarm
  rdlong       tarm,hptr    'get triggerarm parameter
  add          hptr,#4      'increment hub memory pointer     to trigstate
  rdlong       tstate,hptr  'get triggerstate parameter
  add          hptr,#4      'increment hub memory pointer     to period
  rdlong       sper,hptr    'get period parameter
  add          hptr,#4      'increment hub memory pointer     to delay
  rdlong       del,hptr     'get delay parameter
  add          hptr,#4      'increment hub memory pointer     to slt
  rdlong       slt,hptr     'get slt parameter
  add          hptr,#4      'increment hub memory pointer     to sync
  mov          sync,hptr    'save pointer to bufsize
  add          hptr,#4      'increment hub memory pointer     to buffer size
  mov          bsize,hptr   'save pointer to bufsize
  add          hptr,#16      'increment hub memory pointer    to buffer and stay until hub update

  movd         d_inc,#1     'nop store, hub store, etc....
  mov          i,CogBufSz   'get actual buffer size
  jmp          #setuploop   'do nop setup, then capture setup
   
'_______________________________________________________________________________
'

bufpad         long 0 [437]

sper           nop 'period parameter    not used in this module
slt            nop 'slot parameter      not used in this module
sync           nop 'startup sync word   not used in this module

bufend         jmp  #bufferfull         'prototype "jump to bufferfull"

hptr           nop 'save hub memory pointer (initially set to trigmask)
tmask          nop 'get triggermask parameter
tarm           nop 'get triggerarm parameter
tstate         nop 'get triggerstate parameter
del            nop 'get delay parameter
bsize          nop 'save pointer to bufsize
d_inc          nop 'must be nop   'value to increment destination reg by 1 (will be $0000_0200)
i              nop
CogBufSz       long (@bufend-@bufstart)>>2

               fit $1f0


{{
 TERMS OF USE MIT License                                                                                                              
 _______________________________________________________________________________

 Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation     
 files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,    
 modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software
 is furnished to do so, subject to the following conditions                                                                   
                                                                                                                               
 The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
                                                                                                                               
 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE          
 WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR         
 COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,   
 ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                         
 _______________________________________________________________________________
}}