﻿namespace CsharpApplication
{
    using System;
    using System.IO.Ports;
    using System.Runtime.CompilerServices;
    using System.Threading;
    using System.Windows.Forms;

    internal class CommPort
    {
        public bool abortrx;
        private string buffer;
        private SerialPort comport;
        private string conbuffer;
        private bool consoleConnected;
        private CommOptions copts = new CommOptions();
        private MyRegistry myreg = new MyRegistry();
        private TextBox tbconsole;

        public void abort()
        {
            this.comport.Close();
        }

        public void close()
        {
            this.comport.Close();
        }

        public void cycleDTR(int delay)
        {
            try
            {
                bool isOpen = this.comport.IsOpen;
                if (!isOpen)
                {
                    this.open();
                }
                this.comport.DtrEnable = true;
                this.pause(delay);
                this.comport.DtrEnable = false;
                if (!isOpen)
                {
                    this.close();
                }
            }
            catch (Exception exception)
            {
                MessageBox.Show("DTR cycle error " + exception.Message);
            }
        }

        public string expect(string s, int timeout)
        {
            while (timeout-- > 0)
            {
                if (this.buffer.IndexOf(s) >= 0)
                {
                    if (this.consoleConnected)
                    {
                        this.tbconsole.SelectionStart = this.tbconsole.Text.Length;
                        this.tbconsole.ScrollToCaret();
                    }
                    return this.buffer;
                }
                this.pause(1);
            }
            return null;
        }

        public void flush()
        {
            this.buffer = "";
        }

        public string getBuffer()
        {
            return this.buffer;
        }

        public bool init(SerialPort port, TextBox tb)
        {
            this.comport = port;
            this.tbconsole = tb;
            if (this.copts.getCommPort().Length == 0)
            {
                this.comport.PortName = "COM1";
                this.comport.BaudRate = 0x1c200;
                this.comport.Parity = Parity.None;
                this.comport.DataBits = 8;
                this.comport.StopBits = StopBits.One;
                this.comport.Handshake = Handshake.None;
                this.copts = new CommOptions();
                this.copts.TopMost = true;
                this.copts.Show();
                return false;
            }
            this.comport.PortName = this.copts.getCommPort();
            this.comport.BaudRate = this.copts.getSpeed();
            this.comport.Parity = Parity.None;
            this.comport.DataBits = 8;
            this.comport.StopBits = StopBits.One;
            this.comport.Handshake = Handshake.None;
            this.comport.ReadTimeout = 500;
            this.comport.WriteTimeout = 500;
            this.comport.ReadBufferSize = 0x2710;
            return true;
        }

        public void InvokeRxUpdate()
        {
            //int num1 = this.tbconsole.Height / ((int) this.tbconsole.Font.Size);
            this.tbconsole.Text = this.tbconsole.Text + this.conbuffer;
        }

        public bool isConsoleConnected()
        {
            return this.consoleConnected;
        }

        public void open()
        {
            this.comport.Open();
        }

        public void pause(int msdelay)
        {
            int num = msdelay / 0x3e8;
            try
            {
                Application.DoEvents();
                while (num-- > 0)
                {
                    Thread.Sleep(0x3e8);
                    Application.DoEvents();
                    if (!this.comport.IsOpen)
                    {
                        break;
                    }
                }
                Thread.Sleep((int) (msdelay % 0x3e8));
                Application.DoEvents();
            }
            catch (Exception exception)
            {
                Console.WriteLine("Pause error " + exception.Message);
            }
        }

        public void RxData(object sender, SerialDataReceivedEventArgs e)
        {
            string str = "";
            try
            {
                Thread.Sleep(20);
                do
                {
                    str = this.comport.ReadExisting();
                }
                while (this.comport.BytesToRead > 0);
                this.buffer = this.buffer + str;
                if (this.consoleConnected)
                {
                    this.conbuffer = str.Replace("\r", "\r\n");
                    this.tbconsole.BeginInvoke(new InvokeDelegate(this.InvokeRxUpdate));
                }
                Application.DoEvents();
            }
            catch (Exception exception)
            {
                Console.WriteLine(exception.ToString());
            }
        }

        public bool rxReady()
        {
            return (this.buffer.Length > 0);
        }

        public void send(string s)
        {
            this.buffer = "";
            this.abortrx = false;
            if (this.comport.IsOpen)
            {
                this.comport.Write(s);
                if (this.consoleConnected)
                {
                    this.tbconsole.Text = this.tbconsole.Text + s;
                }
            }
        }

        public string sendx(string snd, string exp, int timeout)
        {
            this.buffer = "";
            this.abortrx = false;
            if (this.comport.IsOpen)
            {
                this.comport.Write(snd);
                if (this.consoleConnected)
                {
                    this.tbconsole.Text = this.tbconsole.Text + snd;
                }
                do
                {
                    this.pause(50);
                    if (this.buffer.IndexOf(exp) >= 0)
                    {
                        if (this.consoleConnected)
                        {
                            this.tbconsole.SelectionStart = this.tbconsole.Text.Length;
                            this.tbconsole.ScrollToCaret();
                        }
                        return this.buffer;
                    }
                }
                while (timeout-- > 0);
            }
            return null;
        }

        public void setConsoleConnected(bool val)
        {
            this.consoleConnected = val;
        }

        public string wait(string s)
        {
            while (this.comport.IsOpen)
            {
                this.pause(1);
                if (this.buffer.IndexOf(s) >= 0)
                {
                    if (this.consoleConnected)
                    {
                        this.tbconsole.SelectionStart = this.tbconsole.Text.Length;
                        this.tbconsole.ScrollToCaret();
                    }
                    return this.buffer;
                }
                Application.DoEvents();
            }
            return this.buffer;
        }

        public void waitSend()
        {
            while (this.comport.BytesToWrite > 0)
            {
                Application.DoEvents();
            }
        }

        public delegate void InvokeDelegate();
    }
}

