{{
*************************************
* itoatoi: string/number conversions
*************************************
}}

{
itoatoi.spin lifted from Peter's Format.spin file.

/*
 * @version 1.2 Dec 16, 2007
 * @author Peter Verkaik (verkaik6@zonnet.nl)
 */
}
  
PUB isxdigit(ch): YesNo
  '/**
  ' * Test if character is a hexdigit.
  ' *
  ' * @param ch Character to be tested
  ' * @return True if ch is a hexdigit
  ' */
  YesNo := ((ch =< "9") AND (ch => "0")) OR ((ch =< "F") AND (ch => "A")) OR ((ch =< "f") AND (ch => "a"))

PUB isalpha(ch): YesNo
  '/**
  ' * Test if character is a letter.
  ' *
  ' * @param ch Character to be tested
  ' * @return True if ch is printable
  ' */
  YesNo := ((ch > $41) and (ch < $5B)) or ((ch > $61) and (ch < $7B)) 


PUB isdigit(ch): YesNo
  '/**
  ' * Test if character is a digit.
  ' *
  ' * @param ch Character to be tested
  ' * @return True if ch is a digit
  ' */
  YesNo := (ch =< "9") AND (ch => "0")


PUB isprint(ch): YesNo
  '/**
  ' * Test if character is printable.
  ' *
  ' * @param ch Character to be tested
  ' * @return True if ch is printable
  ' */
  YesNo := (ch & $20) OR (ch & $40) 


PUB isspace(ch): YesNo
  '/**
  ' * Test if character is a space (0x20), tab (\t) or newline (\n).
  ' *
  ' * @param ch Character to be tested
  ' * @return True if ch is a space, tab or newline
  ' */
  YesNo := (ch == 32) OR (ch == 9) OR (ch == 10)


PUB reverse(s) | c,k
  '/**
  ' * Reverse string in place.
  ' *
  ' * @param s Character array to reverse
  ' */
  k := s + strsize(s) - 1 'address of last character
  repeat while (s < k)
    c := byte[s]
    byte[s++] := byte[k]
    byte[k--] := c

PUB itoa(n,s) | sign,sorg
  '/**
  ' * Convert signed integer to signed decimal string.
  ' *
  ' * @param n Integer value to convert
  ' * @param s Character array to hold output
  ' */
  return itoabx(n,s,10,0)


PUB atoi(s): val | sign,n
  '/**
  ' * Convert signed decimal string to signed integer.
  ' *
  ' * @param s Character array that holds decimal string
  ' * @return Value of signed decimal string
  ' */
  return atoib(s,10)

PUB itoab(n,s,b)
  return itoabx(n,s,b,0)

PUB itoabx(n,s,b,c) | lowbit,sorg
  '/*
  ' * Convert unsigned integer to unsigned string using specific base.
  ' * (This is a non-standard function). Used by <code>printf</code>.
  ' *
  ' * @param n Integer value to convert
  ' * @param s Character array to hold output
  ' * @param b Base for output (2=binary, 8=octal, 10=decimal, 16=hexadecimal)
  ' * @returns s pointer
  ' */
  sorg := s
  b := (b >> 1) & $F
  repeat                             '// generate digits/letters in reverse order
    lowbit := n & 1
    n := (n >> 1) & $7FFFFFFF
    byte[s] := ((n // b) << 1) + lowbit
    if (byte[s] < 10)
      byte[s] += "0"
    else
      byte[s] += ("A"-10)
    ++s
    n := n/b
  until (n == 0)
  byte[s] := 0                        '// closing 0
  if c > 0 and strsize(sorg) < c
    repeat c-strsize(sorg)
      byte[s++] := "0" 
    byte[s] := 0
  reverse(sorg)
  return sorg


PUB atoib(s,b): val | n,digit,sorg
  '/*
  ' * Convert unsigned string to unsigned integer using specific base.
  ' * (This is a non-standard function). Used by <code>scanf</code>.
  ' *
  ' * @param s Character array holding unsigned string
  ' * @param b Base for conversion (2=binary, 8=octal, 10=decimal, 16=hexadecimal)
  ' * @return Unsigned integer value
  ' */
  sorg := s
  n := 0
  repeat while isspace(byte[s])
    ++s
  digit := (127 & byte[s++])
  repeat while digit => "0"
    if digit => "a"
      digit -= ("a"-10)
    else
      if digit => "A"
        digit -= ("A"-10)
      else
        digit -= "0"
    if (digit => b)
      quit
    n := (b * n) + digit
    if (byte[s] == 0)
      quit
    digit := (127 & byte[s++])
  return n

  