{{
*************************************
* snprintf module
*************************************
}}
{
The snprintf method is Peter Verkaik's Format.bprintf
implementation with some modifications.
}
OBJ
  sc : "itoatoi"

VAR byte buf[33]

PUB snprintf(str,slen,fmt,arg): rc | pad,len,maxlen,minlen,bi,left,strtype,sorg
{{ 
/**
 * Print formatted string to character buffer, maintaining buffer index.
 * (special version of sprintf, to ease porting printf and sprintf with multiple arguments)
 *
 * format specifiers for byte, word, long: %c,%d,%i,%b,%o,%u,%x
 * format specifier for string: %s
 * field specification (example)
 *   %-05.7d where - for left justify, default right justify
 *                 0 for padding with zeroes, default spaces
 *                 5 minimum field width
 *                 . field separator
 *                 7 maximum field width
 *
 * An original C sprintf statement like:
 * snprintf(buf,buflen,"Today's temperature: low %d high %d\n",12,20);
 * would be written as:
 * OBJ
 *   fmt: "Format"
 * CON
 *   BUFLEN = 128
 * VAR
 *   byte buf[BUFLEN+1]
 *   word k
 * PUB
 *   k := fmt.snprintf(@buf, BUFLEN, string("Today's temperature: low %d high %d"), p(12)+p(20)) )
 *
 * @param str Character array for formatted output
 * @param slen maximum buffer array length
 * @param fmt String defining format
 * @param arg value parameter list as sum of P() calls
 * @return length of string printed to str or 0 if error.
 */
__ snprintf(char* buf, int buflen, char* fmt, void* ...);
}}
  sorg := str 'save start address of output buffer
  len := 0
  repeat while byte[fmt] <> 0

    ' don't let user overrun the specified buffer length
    ' arg concatenation is truncated below to ensure this works
    if strsize(sorg) => slen
      quit                      
      
    if byte[fmt] <> "%"
      if byte[fmt] <> "\"
        byte[str++] := byte[fmt++]
      else
        fmt++ 'skip backslash
        case byte[fmt]
          "\": byte[str++] := "\"
               fmt++
          "t": byte[str++] := $09
               fmt++
          "n": byte[str++] := $0D
               fmt++
          '"r": byte[str++] := $0A ' can't support CR with tv or other objects
          '     fmt++
          other: byte[str++] := "\" 'output skipped backslash
                 byte[str++] := byte[fmt++]
      next
    else
      fmt++ 'skip control character %

    if byte[fmt] == "%"
      byte[str++] := byte[fmt++] '%%
      next
    if byte[fmt] == "-"
      left := true
      ++fmt 'skip -
    else
      left := false
    if byte[fmt] == "0"
      pad := "0"
    else
      pad := " "
    if sc.isdigit(byte[fmt])  '// minimum field width
      bi := 0
      repeat while sc.isdigit(byte[fmt])
        buf[bi++] := byte[fmt++]
      buf[bi] := 0
      minlen := sc.atoi(@buf)
    else
      minlen := 0
    if byte[fmt] == "."   '// maximum field width
      ++fmt 'skip .
      bi := 0
      repeat while sc.isdigit(byte[fmt])
        buf[bi++] := byte[fmt++]
      buf[bi] := 0
      maxlen := sc.atoi(@buf)
    else
      maxlen := 0                           
    strtype := false 'assume no string value
    
    if byte[fmt] ' get arg from queue
      arg := dequeue
      
    case byte[fmt++]
      "c": buf[0] := arg       '// character
           buf[1] := 0
           len := 1
      "d","i": sc.itoa(arg, @buf)  '// signed decimal
               len := strsize(@buf)
      "b": sc.itoab(arg, @buf, 2)  '// binary
           len := strsize(@buf)
      "o": sc.itoab(arg, @buf, 8)  '// octal
           len := strsize(@buf)
      "u": sc.itoab(arg, @buf, 10) '// unsigned decimal
           len := strsize(@buf)
      "x": sc.itoab(arg, @buf, 16) '// hexadecimal
           len := strsize(@buf)
      "f": 'not supported yet  '// 32bit floating point
           buf[0] := 0
           len := 0
      "s": strtype := true
           len := strsize(arg) '// string
      other: buf[0] := 0       '// no valid format specifier
             len := 0

    ' truncate if user is exceeding buffer len
    if strsize(sorg) + len => slen
      len := slen-strsize(sorg)

    if (maxlen <> 0) AND (maxlen < len)
      len := maxlen
    if (minlen > len)
      minlen := minlen - len
    else
      minlen := 0
    bi := 0
    if left == false
      if (buf[bi] == "-") AND (pad == "0")
        byte[str++] := buf[bi++]
        len--
      repeat while minlen > 0
        minlen--
        byte[str++] := pad
    repeat while len > 0
      len--
      if strtype == false
        byte[str++] := buf[bi++]  'copy ascii string of value
      else
        if (byte[arg] == 0)
          quit
        byte[str++] := byte[arg++]  'copy string argument
    if left == true
      repeat while minlen > 0
        minlen--
        byte[str++] := pad

  ' terminate string      
  byte[str] := 0

  rc := str-sorg

  ' check queue
  if count
    rc := 0
    
  ' clean up queue
  flush
    
  return rc

PUB p(v)
{{
This is the "parameter" method. The user will string together a
list of method calls to push parameters onto the print queue.

For example: snprintf(@buf, len, string("values %d %d %d\n"),p(1)+p(2)+p(3))
This will print "values 1 2 3\n" to the buffer.

@param v - value that will replace the printf % format specifier.
@returns 1. Using a list of p(v)+p(v)+p(v) as the arg to snprintf gives arg count. 
}}
  enqueue(v)
  return 1

CON
  QLEN = $ff     ' up to 255 parameters
  
VAR
  long queue[QLEN+1]
  long tail, head
  
PRI enqueue(val)
  long[@queue+head] := val
  head+=4
  head&= QLEN

PRI dequeue : rc
  rc := long[@queue+tail]
  tail+=4
  tail&= QLEN
  return rc
  
PRI count
  if head => tail
    result := head-tail
  else
    result := head+(QLEN-tail)
      
PRI flush
  head := tail

      